@@ -156,7 +156,7 @@ KIOCORE_EXPORT QString KIO::buildErrorString(int errorCode, const QString &error
     case KIO::ERR_CANNOT_DELETE:
         result = i18n("Could not delete file %1.", errorText);
         break;
-    case KIO::ERR_SLAVE_DIED:
+    case KIO::ERR_WORKER_DIED:
         result = i18n("The process for the %1 protocol died unexpectedly.", errorText);
         break;
     case KIO::ERR_OUT_OF_MEMORY:
@@ -206,6 +206,9 @@ KIOCORE_EXPORT QString KIO::buildErrorString(int errorCode, const QString &error
     case KIO::ERR_CANNOT_SYMLINK:
         result = i18n("Could not create symlink %1.\nPlease check permissions.", errorText);
         break;
+    case KIO::ERR_SYMLINKS_NOT_SUPPORTED:
+        result = i18n("Cannot create symlinks at %1.\nThe destination filesystem doesn't support symlinks.", errorText);
+        break;
     case KIO::ERR_NO_CONTENT:
         result = errorText;
         break;
@@ -215,7 +218,9 @@ KIOCORE_EXPORT QString KIO::buildErrorString(int errorCode, const QString &error
     case KIO::ERR_IDENTICAL_FILES:
         result = i18n("The source and destination are the same file.\n%1", errorText);
         break;
-    case KIO::ERR_SLAVE_DEFINED:
+    case KIO::ERR_WORKER_DEFINED:
+        Q_FALLTHROUGH();
+    case KJob::UserDefinedError:
         result = errorText;
         break;
     case KIO::ERR_UPGRADE_REQUIRED:
@@ -236,7 +241,7 @@ KIOCORE_EXPORT QString KIO::buildErrorString(int errorCode, const QString &error
     case KIO::ERR_PASSWD_SERVER:
         result = i18n("Communication with the local password server failed");
         break;
-    case KIO::ERR_CANNOT_CREATE_SLAVE:
+    case KIO::ERR_CANNOT_CREATE_WORKER:
         result = i18n("Unable to create io-slave. %1", errorText);
         break;
     case KIO::ERR_FILE_TOO_LARGE_FOR_FAT32:
@@ -326,8 +331,9 @@ KIOCORE_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &erro
         host = reqUrl->host();
         protocol = reqUrl->scheme();
 
-        if (host.startsWith(QLatin1String("www."))) {
-            domain = host.mid(4);
+        const QLatin1String web("www.");
+        if (host.startsWith(web)) {
+            domain = host.mid(web.size());
         } else {
             domain = host;
         }
@@ -1001,7 +1007,7 @@ KIOCORE_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &erro
         solutions << sAccess << sExists;
         break;
 
-    case KIO::ERR_SLAVE_DIED:
+    case KIO::ERR_WORKER_DIED:
         errorName = i18n("Unexpected Program Termination");
         description = i18n(
             "The program on your computer which provides access "
@@ -1213,8 +1219,8 @@ KIOCORE_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &erro
         solutions << i18n("Drop the item into a different file or folder.");
         break;
 
-    // We assume that the slave has all the details
-    case KIO::ERR_SLAVE_DEFINED:
+    // We assume that the worker has all the details
+    case KIO::ERR_WORKER_DEFINED:
         errorName.clear();
         description = errorText;
         break;
@@ -1235,7 +1241,7 @@ KIOCORE_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &erro
         solutions << i18n("Try restarting your session, or look in the logs for errors from kiod.");
         break;
 
-    case KIO::ERR_CANNOT_CREATE_SLAVE:
+    case KIO::ERR_CANNOT_CREATE_WORKER:
         errorName = i18n("Cannot Initiate the %1 Protocol", protocol);
         techName = i18n("Unable to Create io-slave");
         description = i18n(
