@@ -46,8 +46,7 @@
 
 using namespace KDEPrivate;
 
-struct KUrlNavigatorData
-{
+struct KUrlNavigatorData {
     QUrl rootUrl; // KF6: remove after the deprecated methods have been removed
     QPoint pos; // KF6: remove after the deprecated methods have been removed
     QByteArray state;
@@ -57,7 +56,7 @@ Q_DECLARE_METATYPE(KUrlNavigatorData)
 class KUrlNavigatorPrivate
 {
 public:
-    KUrlNavigatorPrivate(const QUrl& url, KUrlNavigator *qq, KFilePlacesModel *placesModel);
+    KUrlNavigatorPrivate(const QUrl &url, KUrlNavigator *qq, KFilePlacesModel *placesModel);
 
     ~KUrlNavigatorPrivate()
     {
@@ -187,7 +186,7 @@ public:
     } m_subfolderOptions;
 };
 
-KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl& url, KUrlNavigator *qq, KFilePlacesModel *placesModel)
+KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl &url, KUrlNavigator *qq, KFilePlacesModel *placesModel)
     : q(qq)
     , m_coreUrlNavigator(new KCoreUrlNavigator(url, qq))
     , m_showPlacesSelector(placesModel != nullptr)
@@ -198,7 +197,7 @@ KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl& url, KUrlNavigator *qq, K
     q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::currentLocationUrlChanged, q, [this]() {
         Q_EMIT q->urlChanged(m_coreUrlNavigator->currentLocationUrl());
     });
-    q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::currentUrlAboutToChange, q, [this](const QUrl& url) {
+    q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::currentUrlAboutToChange, q, [this](const QUrl &url) {
         Q_EMIT q->urlAboutToBeChanged(url);
     });
     q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::historySizeChanged, q, [this]() {
@@ -210,6 +209,9 @@ KUrlNavigatorPrivate::KUrlNavigatorPrivate(const QUrl& url, KUrlNavigator *qq, K
     q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::historyChanged, q, [this]() {
         Q_EMIT q->historyChanged();
     });
+    q->connect(m_coreUrlNavigator, &KCoreUrlNavigator::urlSelectionRequested, q, [this](const QUrl &url) {
+        Q_EMIT q->urlSelectionRequested(url);
+    });
 
     // initialize the places selector
     q->setAutoFillBackground(false);
@@ -500,15 +502,30 @@ void KUrlNavigatorPrivate::openContextMenu(const QPoint &p)
 
     // We are checking whether the signal is connected because it's odd to have a tab entry even
     // if it's not supported, like in the case of the open dialog
-    if (q->isSignalConnected(QMetaMethod::fromSignal(&KUrlNavigator::tabRequested))) {
-        for (auto button : std::as_const(m_navButtons)) {
-            if (button->geometry().contains(p)) {
-                const auto url = button->url();
-                QAction *openInTab = popup->addAction(QIcon::fromTheme(QStringLiteral("tab-new")), i18n("Open %1 in tab", button->text()));
+    const bool isTabSignal = q->isSignalConnected(QMetaMethod::fromSignal(&KUrlNavigator::tabRequested));
+    const bool isWindowSignal = q->isSignalConnected(QMetaMethod::fromSignal(&KUrlNavigator::newWindowRequested));
+    if (isTabSignal || isWindowSignal) {
+        auto it = std::find_if(m_navButtons.cbegin(), m_navButtons.cend(), [&p](const KUrlNavigatorButton *button) {
+            return button->geometry().contains(p);
+        });
+        if (it != m_navButtons.cend()) {
+            const auto *button = *it;
+            const QUrl url = button->url();
+            const QString text = button->text();
+
+            if (isTabSignal) {
+                QAction *openInTab = popup->addAction(QIcon::fromTheme(QStringLiteral("tab-new")), i18nc("@item:inmenu", "Open \"%1\" in New Tab", text));
                 q->connect(openInTab, &QAction::triggered, q, [this, url]() {
                     Q_EMIT q->tabRequested(url);
                 });
-                break;
+            }
+
+            if (isWindowSignal) {
+                QAction *openInWindow =
+                    popup->addAction(QIcon::fromTheme(QStringLiteral("window-new")), i18nc("@item:inmenu", "Open \"%1\" in New Window", text));
+                q->connect(openInWindow, &QAction::triggered, q, [this, url]() {
+                    Q_EMIT q->newWindowRequested(url);
+                });
             }
         }
     }
@@ -641,9 +658,10 @@ void KUrlNavigatorPrivate::updateButtons(int startIndex)
                     dropUrls(destination, event, button);
                 });
 
-                q->connect(button, &KUrlNavigatorButton::navigatorButtonActivated, q, [this](const QUrl &url, Qt::MouseButton btn, Qt::KeyboardModifiers modifiers) {
+                auto activatedFunc = [this](const QUrl &url, Qt::MouseButton btn, Qt::KeyboardModifiers modifiers) {
                     slotNavigatorButtonClicked(url, btn, modifiers);
-                });
+                };
+                q->connect(button, &KUrlNavigatorButton::navigatorButtonActivated, q, activatedFunc);
 
                 q->connect(button, &KUrlNavigatorButton::finishedTextResolving, q, [this]() {
                     updateButtonVisibility();
@@ -759,9 +777,9 @@ void KUrlNavigatorPrivate::updateButtonVisibility()
     } else {
         // Check whether going upwards is possible. If this is the case, show the drop-down button.
         QUrl url(m_navButtons.front()->url());
-        const bool visible = !url.matches(KIO::upUrl(url), QUrl::StripTrailingSlash)
-                && url.scheme() != QLatin1String("baloosearch")
-                && url.scheme() != QLatin1String("filenamesearch");
+        const bool visible = !url.matches(KIO::upUrl(url), QUrl::StripTrailingSlash) //
+            && url.scheme() != QLatin1String("baloosearch") //
+            && url.scheme() != QLatin1String("filenamesearch");
         m_dropDownButton->setVisible(visible);
     }
 }
