@@ -12,26 +12,26 @@
 #include "../../../pathhelpers_p.h"
 #include "kio_trash.h"
 
-#include <QDataStream>
-#include <QTemporaryFile>
 #include <kprotocolinfo.h>
 
 #include <KConfigGroup>
-#include <QDebug>
+#include <kfileitem.h>
+#include <kio/chmodjob.h>
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
-#include <kio/job.h>
+#include <kio/directorysizejob.h>
 
 #include <KJobUiDelegate>
+
+#include <QDataStream>
+#include <QDebug>
 #include <QDir>
 #include <QFileInfo>
+#include <QStandardPaths>
+#include <QTemporaryFile>
 #include <QUrl>
 #include <QVector>
 
-#include <QStandardPaths>
-#include <kfileitem.h>
-#include <kio/chmodjob.h>
-#include <kio/directorysizejob.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -1180,7 +1180,7 @@ void TestTrash::restoreFileFromSubDir()
     bool ok = job->exec();
     QVERIFY(!ok);
     // dest dir doesn't exist -> error message
-    QCOMPARE(job->error(), KIO::ERR_SLAVE_DEFINED);
+    QCOMPARE(job->error(), KIO::ERR_WORKER_DEFINED);
 
     // check that nothing happened
     QVERIFY(QFile::exists(infoFile));
@@ -1214,7 +1214,7 @@ void TestTrash::restoreFileToDeletedDirectory()
     bool ok = job->exec();
     QVERIFY(!ok);
     // dest dir doesn't exist -> error message
-    QCOMPARE(job->error(), KIO::ERR_SLAVE_DEFINED);
+    QCOMPARE(job->error(), KIO::ERR_WORKER_DEFINED);
 
     // check that nothing happened
     QVERIFY(QFile::exists(infoFile));
