@@ -24,6 +24,7 @@
 #include <QPainter>
 #include <QPushButton>
 #include <QRadioButton>
+#include <QScrollBar>
 #include <QStackedWidget>
 
 #include <KLocalizedString>
@@ -67,15 +68,15 @@ KACLEditWidget::KACLEditWidget(QWidget *parent)
     });
     QVBoxLayout *vbox = new QVBoxLayout();
     hbox->addLayout(vbox);
-    d->m_AddBtn = new QPushButton(i18n("Add Entry..."), this);
+    d->m_AddBtn = new QPushButton(QIcon::fromTheme(QStringLiteral("list-add")), i18nc("@action:button", "Add..."), this);
     vbox->addWidget(d->m_AddBtn);
     d->m_AddBtn->setObjectName(QStringLiteral("add_entry_button"));
     connect(d->m_AddBtn, &QAbstractButton::clicked, d->m_listView, &KACLListView::slotAddEntry);
-    d->m_EditBtn = new QPushButton(i18n("Edit Entry..."), this);
+    d->m_EditBtn = new QPushButton(QIcon::fromTheme(QStringLiteral("document-edit")), i18nc("@action:button", "Edit..."), this);
     vbox->addWidget(d->m_EditBtn);
     d->m_EditBtn->setObjectName(QStringLiteral("edit_entry_button"));
     connect(d->m_EditBtn, &QAbstractButton::clicked, d->m_listView, &KACLListView::slotEditEntry);
-    d->m_DelBtn = new QPushButton(i18n("Delete Entry"), this);
+    d->m_DelBtn = new QPushButton(QIcon::fromTheme(QStringLiteral("list-remove")), i18nc("@action:button", "Delete"), this);
     vbox->addWidget(d->m_DelBtn);
     d->m_DelBtn->setObjectName(QStringLiteral("delete_entry_button"));
     connect(d->m_DelBtn, &QAbstractButton::clicked, d->m_listView, &KACLListView::slotRemoveEntry);
@@ -83,10 +84,7 @@ KACLEditWidget::KACLEditWidget(QWidget *parent)
     d->_k_slotUpdateButtons();
 }
 
-KACLEditWidget::~KACLEditWidget()
-{
-    delete d;
-}
+KACLEditWidget::~KACLEditWidget() = default;
 
 void KACLEditWidget::KACLEditWidgetPrivate::_k_slotUpdateButtons()
 {
@@ -608,23 +606,6 @@ KACLListView::KACLListView(QWidget *parent)
     header()->setSectionResizeMode(QHeaderView::ResizeToContents);
     setRootIsDecorated(false);
 
-    // fill the lists of all legal users and groups
-    struct passwd *user = nullptr;
-    setpwent();
-    while ((user = getpwent()) != nullptr) {
-        m_allUsers << QString::fromLatin1(user->pw_name);
-    }
-    endpwent();
-
-    struct group *gr = nullptr;
-    setgrent();
-    while ((gr = getgrent()) != nullptr) {
-        m_allGroups << QString::fromLatin1(gr->gr_name);
-    }
-    endgrent();
-    m_allUsers.sort();
-    m_allGroups.sort();
-
     connect(this, &QTreeWidget::itemClicked, this, &KACLListView::slotItemClicked);
 
     connect(this, &KACLListView::itemDoubleClicked, this, &KACLListView::slotItemDoubleClicked);
@@ -634,8 +615,25 @@ KACLListView::~KACLListView()
 {
 }
 
+QSize KACLListView::sizeHint() const
+{
+    const int width = header()->length() + verticalScrollBar()->width();
+    const int height = 7 * rowHeight(model()->index(0, 0, QModelIndex()));
+    return {width, height};
+}
+
 QStringList KACLListView::allowedUsers(bool defaults, KACLListViewItem *allowedItem)
 {
+    if (m_allUsers.isEmpty()) {
+        struct passwd *user = nullptr;
+        setpwent();
+        while ((user = getpwent()) != nullptr) {
+            m_allUsers << QString::fromLatin1(user->pw_name);
+        }
+        endpwent();
+        m_allUsers.sort();
+    }
+
     QStringList allowedUsers = m_allUsers;
     QTreeWidgetItemIterator it(this);
     while (*it) {
@@ -654,6 +652,16 @@ QStringList KACLListView::allowedUsers(bool defaults, KACLListViewItem *allowedI
 
 QStringList KACLListView::allowedGroups(bool defaults, KACLListViewItem *allowedItem)
 {
+    if (m_allGroups.isEmpty()) {
+        struct group *gr = nullptr;
+        setgrent();
+        while ((gr = getgrent()) != nullptr) {
+            m_allGroups << QString::fromLatin1(gr->gr_name);
+        }
+        endgrent();
+        m_allGroups.sort();
+    }
+
     QStringList allowedGroups = m_allGroups;
     QTreeWidgetItemIterator it(this);
     while (*it) {
