@@ -7,23 +7,23 @@
 
 #include "paste.h"
 #include "kio_widgets_debug.h"
-#include "pastedialog_p.h"
 
 #include "../pathhelpers_p.h"
 #include "kio/copyjob.h"
 #include "kio/deletejob.h"
 #include "kio/global.h"
-#include "kio/job.h"
 #include "kio/renamedialog.h"
-#include "kprotocolmanager.h"
+#include "kio/statjob.h"
+#include "pastedialog_p.h"
+#include <kdirnotify.h>
+#include <kfileitem.h>
+#include <kfileitemlistproperties.h>
+#include <kio/storedtransferjob.h>
 
 #include <KJobWidgets>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KUrlMimeData>
-#include <kdirnotify.h>
-#include <kfileitem.h>
-#include <kfileitemlistproperties.h>
 
 #include <QApplication>
 #include <QClipboard>
@@ -157,10 +157,10 @@ static QByteArray chooseFormatAndUrl(const QUrl &u,
 
     const QString chosenFormat = formats[dlg.comboItem()];
     if (clipboard && !qApp->clipboard()->mimeData()->hasFormat(chosenFormat)) {
-        KMessageBox::sorry(widget,
-                           i18n("The clipboard has changed since you used 'paste': "
-                                "the chosen data format is no longer applicable. "
-                                "Please copy again what you wanted to paste."));
+        KMessageBox::information(widget,
+                                 i18n("The clipboard has changed since you used 'paste': "
+                                      "the chosen data format is no longer applicable. "
+                                      "Please copy again what you wanted to paste."));
         return QByteArray();
     }
 
