@@ -1,20 +1,8 @@
-/*  This file is part of the KDE libraries
-    Copyright (C) 2009 David Nolden <david.nolden.kdevelop@art-master.de>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2009 David Nolden <david.nolden.kdevelop@art-master.de>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef _THUMBSEQUENCECREATOR_H_
@@ -24,6 +12,10 @@
 
 #include <qglobal.h>
 
+#include <memory>
+
+class ThumbSequenceCreatorPrivate;
+
 /**
  * @class ThumbSequenceCreator thumbsequencecreator.h <KIO/ThumbSequenceCreator>
  *
@@ -47,7 +39,7 @@ class KIOWIDGETS_EXPORT ThumbSequenceCreator : public ThumbCreator
 public:
     Q_DISABLE_COPY(ThumbSequenceCreator)
     ThumbSequenceCreator();
-    virtual ~ThumbSequenceCreator();
+    ~ThumbSequenceCreator() override;
 
     /**
      * If this thumb-creator can create a sequence of thumbnails,
@@ -73,9 +65,46 @@ public:
      */
     void setSequenceIndex(float index);
 
+    /**
+     * Returns the point at which this thumb-creator's sequence indices
+     * will wrap around (loop).
+     *
+     * Usually, the frontend will call setSequenceIndex() with indices
+     * that increase indefinitely with time, e.g. as long as the user
+     * keeps hovering a video file. Most thumb-creators however only
+     * want to display a finite sequence of thumbs, after which their
+     * sequence repeats.
+     *
+     * This method can return the sequence index at which this
+     * thumb-creator's sequence starts wrapping around to the start
+     * again ("looping"). The frontend may use this to generate only
+     * thumbs up to this index, and then use cached versions for the
+     * repeating sequence instead.
+     *
+     * Like sequenceIndex(), fractional values can be used if the
+     * wraparound does not happen at an integer position, but
+     * frontends handling only integer sequence indices may choose
+     * to round it down.
+     *
+     * By default, this method returns a negative index, which signals
+     * the frontend that it can't rely on this fixed-length sequence.
+     *
+     * @since 5.80
+     */
+    float sequenceIndexWraparoundPoint() const;
+
+protected:
+    /**
+     * Sets the point at which this thumb-creator's sequence indices
+     * will wrap around.
+     *
+     * @see sequenceIndexWraparoundPoint()
+     * @since 5.80
+     */
+    void setSequenceIndexWraparoundPoint(float wraparoundPoint);
+
 private:
-    class Private;
-    Private *d;
+    std::unique_ptr<ThumbSequenceCreatorPrivate> d;
 };
 
 typedef ThumbCreator *(*newCreator)();
